c     Programme DIPOLE2.FOR.  (December 1988)
C     This programme calculates the scattering matrix elements
C     for an oriented or orientationally averaged particle. The averaging
C     is done by first obtaining the inverse of the interaction matrix for
C     a single orientation and using this inverse to calculate scattering
C     at other orientations.
C     Dimensioned for 200 dipoles and can be used for real refractive indices
C     only. Can be easily modified for complex polarizabilities.
c     Feb. 1990.  Dimensioned for 450 dipoles.

c     Mar. 1990.  Modified from DIPOLE2.FOR to do sum over wavelength.
c     Added segment for rectangular block.
c     May 1990. Dimensioned for 300 dipoles.

c     Jun 1990.  Modified to do two particles as a function of distance.


      COMPLEX A(300,300), B(300,300), CI, XA(900,900), XB(900)
      COMPLEX Z(900),DET(2),XIK(3),YIK,AEL(3),AER(3),PXB(900)
      COMPLEX REX,DEX,XC(900),PXC(900),DR(3),DL(3),AL,BL,CL
      COMPLEX AR,BR,CR,ELS,EDS,PELS,PEDS
      COMPLEX PRL1,PRL2,PRL3,PRR1,PRR2,PRR3
      COMPLEX S1,S2,S3,S4,EXK
c      COMPLEX S10,S20,S30,S40
      INTEGER IPVT(900)
      DIMENSION S(4,4),SB(400),CB(400)
      DIMENSION CT1(100),ST1(100),CF1(100),SF1(100)
      DIMENSION OX(300),OY(300),OZ(300),RJK(300,300),OAL4(300,3,3)
      DIMENSION S11(400),S12(400),S13(400),S14(400),S22(400)
      DIMENSION S23(400),S24(400),S33(400),S34(400),S44(400)
      DIMENSION OAT(3,3),OEA(3,3),RAT(3,3),REA(3,3)
      DIMENSION AL1(3,3),AL2(3,3),AL3(3,3)
      DIMENSION DN(3),DT(3),DP(3)
      DIMENSION UTEN(3,3),ALPH(100)
      COMMON /DATA/R,P,IH1,RSM,PSM,IH2
 
C     Data file (in file assigned to FOR015) should contain the following:
C     NOC - 1 for no interactions; 2 for inclusion of interactions.
 
      READ(15,*) NOC
      WRITE(6,*) NOC

C     R,P - Radius and pitch of simple helix or large helix in superhelix.
C     NSM - number of dipoles per turn of a simple helix or per turn in the
C     small helix of a superhelix.
C     RM - number of turns of helix.
C     IH1 - +1 for right handed and -1 for left handed helix. Specified 
C     for simple helix or large helix of superhelix.

      READ(15,*) R,P,NSM,RM,IH1
      WRITE(6,*) R,P,NSM,RM,IH1

C     RSM,PSM - Radius and pitch of small helix in superhelix. For a simple
C     helix set RSM equal to zero.
C     IH2 - Handedness of small helix in superhelix - +1 for right and -1
C     for left.

      READ(15,*) RSM,PSM,IH2
      WRITE(6,*) RSM,PSM,IH2

C     XLAMDA - wavelength of incident light.
C     DFI - azimuthal angle.
C     NN - even number of scattering angles (0 to 360) at which the matrix 
C     elements are to be calculated. For average change NM in programme so
C     that only scattering angles (0 to 180) are used.
C     NS - number of angle psi (0 to 2*pi) for orientation average.
C     NT - number of angles theta (0 to pi) for orientation average.
C     NF - number of angles phi (0 to 2*pi) for orientation average.

      READ(15,*) XLAMDA,DFI,NN,NS,NT,NF
      WRITE(6,*) XLAMDA,DFI,NN,NS,NT,NF

C     ALPH1,ALPH2,ALPH3 - real polarizabilities along normal, tangent
C     and perpendicular of helix.

      READ(15,*) ALPH1,ALPH2,ALPH3
      WRITE(6,*) ALPH1,ALPH2,ALPH3

C     OFI,OTHET,OSI - Euler angles (phi,theta,psi) which specify rotation
C     of the particle from the original orientation.

      READ(15,*) OFI,OTHET,OSI
      WRITE(6,*) OFI,OTHET,OSI

      READ(15,*) ISHAPE,LSIZE,FSZ,NHEI,NWID
      WRITE(6,*) ISHAPE,LSIZE,FSZ,NHEI,NWID

      READ(15,*) XLF0,XLFINC,NXLF
      WRITE(6,*) XLF0,XLFINC,NXLF

      READ(15,*) RSEP
      WRITE(6,*) RSEP

      DO 511 I=1,NXLF
      READ(15,*) ALPH(I)
      WRITE(6,*) ALPH(I)
511   CONTINUE



      CI=(0.0,1.0)
      PI=3.1415926536
      XK=(2.0*PI/XLAMDA)
      XKF=(XK**6)/4.0
      XKF2=SQRT(XKF)

      DSI=2.0*PI/FLOAT(NS)
      dtheta=pi/float(nt)
c      DTHETA=2.0*PI/FLOAT(NT)
      DEFI=PI/FLOAT(NF)
      STF2=(SIN(DTHETA/2.0)*SIN(DEFI/2.0))/(PI*NS)
      DBETA=PI/FLOAT(NN)
      DFI=DFI*PI/180.0
      CFI=COS(DFI)
      SFI=SIN(DFI)
      NM=NN+1

      XLF=XLF0
      DO 379 JK=1,NXLF
      ALPH1=ALPH(JK)

      IF (ISHAPE.EQ.2) GO TO 250
      IF (ISHAPE.EQ.3) GO TO 350
      IF (ISHAPE.EQ.4) GO TO 450

C     Set up position coordinates and polarizabilities for superhelix or
C     simple helix.

      ANG=0.0
      ANGP=0.0
      DANGP=2.0*PI/NSM
      YM=R**2+(P**2)/(4.0*(PI**2))
      YM=SQRT(YM)
      YM2=2.0*PI*YM
      FNSM=FLOAT(NSM)
      IF (RSM .EQ. 0.0) PSM=YM2
      IF (RSM .EQ. 0.0) DANGP=0.0
      DANG=PSM/(FNSM*YM)
      TURNS=YM2/PSM
      IF (RSM .EQ. 0.0) TURNS=1.0
      TURNS=RM*TURNS
      TOTAL=TURNS*NSM
      ITOT=INT(TOTAL)
      WRITE(6,*) TURNS
      WRITE(6,*) ITOT
      CRM=R/YM
      CPM=P/YM2
      KM=3*ITOT
 

      DO 10 I=1,ITOT
      C1=COS(ANG)
      SA1=SIN(ANG)
      CA2=COS(ANGP)
      SA2=SIN(ANGP)
      C2=IH1*SA1
      C6=-IH1*CPM*SA1
      C7=CPM*C1
      C8=-IH1*CRM
      OX(I)=C1*RSM*CA2+IH2*C6*RSM*SA2+R*C1
      OY(I)=C2*RSM*CA2+IH2*C7*RSM*SA2+IH1*R*SA1
      OZ(I)=IH2*RSM*SA2*C8+P*ANG/(2.0*PI)
      DN(1)=C1*CA2+IH2*SA2*C6
      DN(2)=C2*CA2+C7*IH2*SA2
      DN(3)=IH2*SA2*C8
      CALL TANG(YM,YM2,C1,SA1,CA2,SA2,DT)
      DP(1)=DT(2)*DN(3)-DT(3)*DN(2)
      DP(2)=DT(3)*DN(1)-DT(1)*DN(3)
      DP(3)=DT(1)*DN(2)-DT(2)*DN(1)
      DO 100 MB=1,3
        DO 101 MC=1,3
        AL1(MB,MC)=ALPH1*DN(MB)*DN(MC)+ALPH2*DT(MB)*DT(MC)+
     1  ALPH3*DP(MB)*DP(MC)
        OAL4(I,MB,MC)=AL1(MB,MC)
101     CONTINUE
100   CONTINUE
      ANG=ANG+DANG
      ANGP=ANGP+DANGP
10    CONTINUE
      GO TO 251      


250   L2=LSIZE**2
      FL2=FSZ*L2
      ISPH=0
      ISPH1=0

      DO 512 NB=1,3
        DO 513 NC=1,3
        UTEN(NB,NC)=0.0
513     CONTINUE
      UTEN(NB,NB)=1.0
512   CONTINUE

      DO 261 IZ=-LSIZE,LSIZE,1
      DO 262 IY=-LSIZE,LSIZE,1
      DO 263 IX=-LSIZE,LSIZE,1

      RDIS=1.0*(IX**2+IY**2+IZ**2)
      IF (RDIS .GT. FL2) GO TO 263
      ISPH=ISPH+1
      ISPH1=ISPH1+1
      OX(ISPH)=1.0*IX
      OY(ISPH)=1.0*IY
      OZ(ISPH)=1.0*IZ

C      RDIS=1.0*(OX(ISPH)**2+OY(ISPH)**2+OZ(ISPH)**2)
C      IF (RDIS .GT. FL2) GO TO 263
C      ISPH=ISPH+1

      DO 514 MB=1,3
        DO 515 MC=1,3
        OAL4(ISPH,MB,MC)=ALPH1*UTEN(MB,MC)
515     CONTINUE
514   CONTINUE

C       WRITE(17,*) ISPH,OX(ISPH),OY(ISPH),OZ(ISPH),RDIS
C      WRITE(18,*) ((OAL4(ISPH,MB,MC),MB=1,3),MC=1,3)

263   CONTINUE
262   CONTINUE
261   CONTINUE

      DO 861 IZ=-LSIZE,LSIZE,1
      DO 862 IY=-LSIZE,LSIZE,1
      DO 863 IX=-LSIZE,LSIZE,1

      RDIS=1.0*(IX**2+IY**2+IZ**2)
      IF (RDIS .GT. FL2) GO TO 863
      ISPH=ISPH+1
      OX(ISPH)=1.0*IX
      OY(ISPH)=1.0*IY
      OZ(ISPH)=1.0*IZ + RSEP

C      RDIS=1.0*(OX(ISPH)**2+OY(ISPH)**2+OZ(ISPH)**2)
C      IF (RDIS .GT. FL2) GO TO 263
C      ISPH=ISPH+1

      DO 914 MB=1,3
        DO 915 MC=1,3
        OAL4(ISPH,MB,MC)=ALPH1*UTEN(MB,MC)
915     CONTINUE
914   CONTINUE

C       WRITE(17,*) ISPH,OX(ISPH),OY(ISPH),OZ(ISPH),RDIS
C      WRITE(18,*) ((OAL4(ISPH,MB,MC),MB=1,3),MC=1,3)

863   CONTINUE
862   CONTINUE
861   CONTINUE


           ISPH2=ISPH1+1

            OFI=OFI*PI/180.0
            OCF=COS(OFI)
            OSF=SIN(OFI)
            OTHET=OTHET*PI/180.0
            OCT=COS(OTHET)
            OST=SIN(OTHET)
            OSI=OSI*PI/180.0
            OCS=COS(OSI)
            OSS=SIN(OSI)
        CALL EULERT(OCF,OSF,OCS,OSS,OCT,OST,OAT,OEA)
        DO 481 LB=ISPH2,ISPH
        EX=OAT(1,1)*OX(LB)+OAT(1,2)*OY(LB)+OAT(1,3)*OZ(LB)
        EY=OAT(2,1)*OX(LB)+OAT(2,2)*OY(LB)+OAT(2,3)*OZ(LB)
        EZ=OAT(3,1)*OX(LB)+OAT(3,2)*OY(LB)+OAT(3,3)*OZ(LB)
        OX(LB)=EX
        OY(LB)=EY
        OZ(LB)=EZ
        DO 482 LC=1,3
          DO 483 LD=1,3
          AL2(LC,LD)=OAL4(LB,LC,LD)
483       CONTINUE
482     CONTINUE
        CALL TENROT(AL2,OAT,OEA,AL3)
        DO 484 LE=1,3
          DO 485 LF=1,3
          OAL4(LB,LE,LF)=AL3(LE,LF)
485       CONTINUE
484     CONTINUE
481      CONTINUE

      GO TO 252



350   L2=LSIZE**2
      FL2=FSZ*L2
      ISPH=0
      ISPH1=0
      IDSK=0

      DO 516 NB=1,3
        DO 517 NC=1,3
        UTEN(NB,NC)=0.0
517     CONTINUE
      UTEN(NB,NB)=1.0
516   CONTINUE

      DO 361 IZ=1,NHEI
      DO 362 IY=-LSIZE,LSIZE,1
      DO 363 IX=-LSIZE,LSIZE,1

      RDIS=1.0*(IX**2+IY**2)
      IF (RDIS .GT. FL2) GO TO 363
      IF (IZ .EQ. 1) IDSK=IDSK+1
      ISPH=ISPH+1
      ISPH1=ISPH1+1
      OX(ISPH)=1.0*IX
      OY(ISPH)=1.0*IY
      OZ(ISPH)=1.0*IZ

      DO 518 MB=1,3
        DO 519 MC=1,3
        OAL4(ISPH,MB,MC)=ALPH1*UTEN(MB,MC)
519     CONTINUE
518   CONTINUE

C       WRITE(17,*) ISPH,OX(ISPH),OY(ISPH),OZ(ISPH),RDIS
C      WRITE(18,*) ((OAL4(ISPH,MB,MC),MB=1,3),MC=1,3)

363   CONTINUE
362   CONTINUE
361   CONTINUE

      DO 761 IZ=1,NHEI
      DO 762 IY=-LSIZE,LSIZE,1
      DO 763 IX=-LSIZE,LSIZE,1

      RDIS=1.0*(IX**2+IY**2)
      IF (RDIS .GT. FL2) GO TO 763
      IF (IZ .EQ. 1) IDSK=IDSK+1
      ISPH=ISPH+1
      OX(ISPH)=1.0*IX
      OY(ISPH)=1.0*IY
      OZ(ISPH)=1.0*IZ + RSEP

      DO 918 MB=1,3
        DO 919 MC=1,3
        OAL4(ISPH,MB,MC)=ALPH1*UTEN(MB,MC)
919     CONTINUE
918   CONTINUE

C       WRITE(17,*) ISPH,OX(ISPH),OY(ISPH),OZ(ISPH),RDIS
C      WRITE(18,*) ((OAL4(ISPH,MB,MC),MB=1,3),MC=1,3)

763   CONTINUE
762   CONTINUE
761   CONTINUE

           ISPH2=ISPH1+1

            OFI=OFI*PI/180.0
            OCF=COS(OFI)
            OSF=SIN(OFI)
            OTHET=OTHET*PI/180.0
            OCT=COS(OTHET)
            OST=SIN(OTHET)
            OSI=OSI*PI/180.0
            OCS=COS(OSI)
            OSS=SIN(OSI)
        CALL EULERT(OCF,OSF,OCS,OSS,OCT,OST,OAT,OEA)
        DO 411 LB=ISPH2,ISPH
        EX=OAT(1,1)*OX(LB)+OAT(1,2)*OY(LB)+OAT(1,3)*OZ(LB)
        EY=OAT(2,1)*OX(LB)+OAT(2,2)*OY(LB)+OAT(2,3)*OZ(LB)
        EZ=OAT(3,1)*OX(LB)+OAT(3,2)*OY(LB)+OAT(3,3)*OZ(LB)
        OX(LB)=EX
        OY(LB)=EY
        OZ(LB)=EZ
        DO 402 LC=1,3
          DO 403 LD=1,3
          AL2(LC,LD)=OAL4(LB,LC,LD)
403       CONTINUE
402     CONTINUE
        CALL TENROT(AL2,OAT,OEA,AL3)
        DO 404 LE=1,3
          DO 405 LF=1,3
          OAL4(LB,LE,LF)=AL3(LE,LF)
405       CONTINUE
404     CONTINUE
411      CONTINUE

      GO TO 252


450   ISPH=0

      DO 520 NB=1,3
        DO 521 NC=1,3
        UTEN(NB,NC)=0.0
521     CONTINUE
      UTEN(NB,NB)=1.0
520   CONTINUE

      DO 461 IZ=1,NHEI
      DO 462 IY=1,NWID
      DO 463 IX=1,LSIZE

      ISPH=ISPH+1
      OX(ISPH)=1.0*IX
      OY(ISPH)=1.0*IY
      OZ(ISPH)=1.0*IZ

      DO 522 MB=1,3
        DO 523 MC=1,3
        OAL4(ISPH,MB,MC)=ALPH1*UTEN(MB,MC)
523     CONTINUE
522   CONTINUE

C       WRITE(17,*) ISPH,OX(ISPH),OY(ISPH),OZ(ISPH),RDIS
C      WRITE(18,*) ((OAL4(ISPH,MB,MC),MB=1,3),MC=1,3)

463   CONTINUE
462   CONTINUE
461   CONTINUE


252      FSPH=3./(4.*PI)
      RADSPH=(FSPH*ISPH)**.333333
      DIASPH=2.*RADSPH
      ITOT=ISPH
      KM=3*ISPH

      RADDSK=SQRT(IDSK/PI)


       DO 612 I=1,ISPH
       WRITE(17,*) I,OX(I),OY(I),OZ(I)
       WRITE(18,*) ((OAL4(I,MB,MC),MB=1,3),MC=1,3)
612    CONTINUE

251   GO TO 255
c     Rotate particle to desired orientation

            OFI=OFI*PI/180.0
            OCF=COS(OFI)
            OSF=SIN(OFI)
            OTHET=OTHET*PI/180.0
            OCT=COS(OTHET)
            OST=SIN(OTHET)
            OSI=OSI*PI/180.0
            OCS=COS(OSI)
            OSS=SIN(OSI)
        CALL EULERT(OCF,OSF,OCS,OSS,OCT,OST,OAT,OEA)
        DO 11 LB=1,ITOT
        EX=OAT(1,1)*OX(LB)+OAT(1,2)*OY(LB)+OAT(1,3)*OZ(LB)
        EY=OAT(2,1)*OX(LB)+OAT(2,2)*OY(LB)+OAT(2,3)*OZ(LB)
        EZ=OAT(3,1)*OX(LB)+OAT(3,2)*OY(LB)+OAT(3,3)*OZ(LB)
        OX(LB)=EX
        OY(LB)=EY
        OZ(LB)=EZ
        DO 102 LC=1,3
          DO 103 LD=1,3
          AL2(LC,LD)=OAL4(LB,LC,LD)
103       CONTINUE
102     CONTINUE
        CALL TENROT(AL2,OAT,OEA,AL3)
        DO 104 LE=1,3
          DO 105 LF=1,3
          OAL4(LB,LE,LF)=AL3(LE,LF)
105       CONTINUE
104     CONTINUE
11      CONTINUE

255   CONTINUE

C     Do 379 is loop over wavelength (equivalent to changing size of particle)

C      XLF=XLF0
C      DO 379 JK=1,NXLF

      xlamda0=XLF*radsph
      XLAMDA=(XLF/1.5)*RADSPH
      XK=(2.*PI)/XLAMDA
      XKF=(XK**6)/4.
      XKF2=SQRT(XKF)

      WRITE(26,*) ISPH,radsph,XLAMDA0,NM
      WRITE(6,*) ISPH,radsph,XLAMDA0,NM
      write(26,*) ishape,lsize,fsz,nhei,RADDSK
 

C     Calculate interaction terms A(J,K) and B(J,K) for all pairs in the
C     collection.

      ITOT1=ITOT-1
      DO 20 J=1,ITOT1
      J1=J+1
      DO 30 K=J1,ITOT
      DX2=(OX(J)-OX(K))**2
      DY2=(OY(J)-OY(K))**2
      DZ2=(OZ(J)-OZ(K))**2
      RJK(J,K)=SQRT(DX2+DY2+DZ2)
      RJK(K,J)=RJK(J,K)
      XR=RJK(J,K)
      EXK=CEXP(CI*XK*XR)/XR
      A(J,K)=(XK**2-1.0/(XR**2)+CI*XK/XR)*EXK
      B(J,K)=(3.0/(XR**2)-XK**2-3.0*CI*XK/XR)*EXK
      A(K,J)=A(J,K)
      B(K,J)=B(J,K)
30    CONTINUE
20    CONTINUE
 
 
      BETA=0.0
      DO 106 LK=1,NM
        SB(LK)=SIN(BETA)
        CB(LK)=COS(BETA)
        BETA=BETA+DBETA
106   CONTINUE

      cexts=0.0
 
      DO 107 LA=1,NM
        S11(LA)=0.0
C        S12(LA)=0.0
C        S13(LA)=0.0
C        S14(LA)=0.0
C        S22(LA)=0.0
C        S23(LA)=0.0
C        S24(LA)=0.0
C        S33(LA)=0.0
C        S34(LA)=0.0
C        S44(LA)=0.0
107   CONTINUE
 
      AFI=0.0
      DO 108 NE=1,NF
      CF1(NE)=COS(AFI)
      SF1(NE)=SIN(AFI)
      AFI=AFI+DEFI
108   CONTINUE
 
      ATHETA=0.0
      DO 109 ND=1,NT
        GAM=ATHETA-DTHETA/2.0
        IF (NT .EQ. 1) GAM=ATHETA
        CT1(ND)=COS(GAM)
        ST1(ND)=SIN(GAM)
        ATHETA=ATHETA+DTHETA
109   CONTINUE
 
C     Set up 3Nx3N interaction matrix (XA) and 3Nx1 incident field 
C     vectors (XB and PXB).

      DO 110 JI=1,KM
        DO 111 JJ=1,KM
        XA(JI,JJ)=(0.0,0.0)
111     CONTINUE
        XA(JI,JI)=(1.0,0.0)
110   CONTINUE

      IF (NOC .EQ. 1) GO TO 56

      DO 50 II=1,ITOT
      DO 60 IL=1,3
      IT=3*(II-1)+IL
      DO 70 IJ=1,ITOT
      IF (IJ .EQ. II) GO TO 70
      IV=3*(IJ-1)+1
      IW=3*(IJ-1)+2
      IX=3*(IJ-1)+3
      XN=(OX(II)-OX(IJ))/RJK(II,IJ)
      YN=(OY(II)-OY(IJ))/RJK(II,IJ)
      ZN=(OZ(II)-OZ(IJ))/RJK(II,IJ)
      ALN1=OAL4(IJ,1,1)*XN+OAL4(IJ,2,1)*YN+OAL4(IJ,3,1)*ZN
      ALN2=OAL4(IJ,1,2)*XN+OAL4(IJ,2,2)*YN+OAL4(IJ,3,2)*ZN
      ALN3=OAL4(IJ,1,3)*XN+OAL4(IJ,2,3)*YN+OAL4(IJ,3,3)*ZN
      IF (IL-2) 71,72,73
71    XA(IT,IV)=-(A(II,IJ)*OAL4(IJ,1,1)+B(II,IJ)*ALN1*XN)
      XA(IT,IW)=-(A(II,IJ)*OAL4(IJ,1,2)+B(II,IJ)*ALN2*XN)
      XA(IT,IX)=-(A(II,IJ)*OAL4(IJ,1,3)+B(II,IJ)*ALN3*XN)
      GO TO 70
72    XA(IT,IV)=-(A(II,IJ)*OAL4(IJ,2,1)+B(II,IJ)*ALN1*YN)
      XA(IT,IW)=-(A(II,IJ)*OAL4(IJ,2,2)+B(II,IJ)*ALN2*YN)
      XA(IT,IX)=-(A(II,IJ)*OAL4(IJ,2,3)+B(II,IJ)*ALN3*YN)
      GO TO 70
73    XA(IT,IV)=-(A(II,IJ)*OAL4(IJ,3,1)+B(II,IJ)*ALN1*ZN)
      XA(IT,IW)=-(A(II,IJ)*OAL4(IJ,3,2)+B(II,IJ)*ALN2*ZN)
      XA(IT,IX)=-(A(II,IJ)*OAL4(IJ,3,3)+B(II,IJ)*ALN3*ZN)
70    CONTINUE
60    CONTINUE
50    CONTINUE

C     Calls to CGECO and CGEDI calculate the inverse of the
C     matrix XA and store it in XA.
 
      CALL CGECO(XA,900,KM,IPVT,RCOND,Z)
      CALL CGEDI(XA,900,KM,IPVT,DET,Z,01)
 
56    YIK=CI*XK
 
C     Loops (17,18 and 19) to calculate numerically the orientation average 
C     of the scattering matrix elements using the inverse of the 
C     interaction matrix.

      SAREA=0.0
      ASI=0.0
      DO 17 NA=1,NS
      CS=COS(ASI)
      SS=SIN(ASI)
 
      ATHETA=0.0
      DO 18 NB=1,NT
      CT=CT1(NB)
      ST=ST1(NB)
      AREAU=ABS(ST1(NB)*STF2)
 
      AFI=0.0
      DO 19 NC=1,NF
      CF=CF1(NC)
      SF=SF1(NC)
      CALL EULERT(CF,SF,CS,SS,CT,ST,RAT,REA)
      IF (NT .EQ. 1) AREAU=1.0
 
      XIK(1)=YIK*REA(1,3)
      XIK(2)=YIK*REA(2,3)
      XIK(3)=YIK*REA(3,3)

      AEL(1)=REA(1,1)+REA(1,2)*CI
      AEL(2)=REA(2,1)+REA(2,2)*CI
      AEL(3)=REA(3,1)+REA(3,2)*CI
      AER(1)=REA(1,1)-REA(1,2)*CI
      AER(2)=REA(2,1)-REA(2,2)*CI
      AER(3)=REA(3,1)-REA(3,2)*CI

      DO 112 IJ=1,ITOT
         REX=CEXP(XIK(1)*OX(IJ)+XIK(2)*OY(IJ)+XIK(3)*OZ(IJ))
         IV=3*(IJ-1)+1
         IW=3*(IJ-1)+2
         IX=3*(IJ-1)+3
         XB(IV)=AEL(1)*REX
         XB(IW)=AEL(2)*REX
         XB(IX)=AEL(3)*REX
         PXB(IV)=AER(1)*REX
         PXB(IW)=AER(2)*REX
         PXB(IX)=AER(3)*REX
112   CONTINUE

      IF (NOC .EQ. 1) GO TO 57
 
          DO 113 L1=1,KM
          XC(L1)=(0.0,0.0)
          PXC(L1)=(0.0,0.0)
             DO 114 L2=1,KM
             XC(L1)=XC(L1)+XA(L1,L2)*XB(L2)
             PXC(L1)=PXC(L1)+XA(L1,L2)*PXB(L2)
114          CONTINUE
113       CONTINUE
          GO TO 58

57    DO 115 LN=1,KM
      XC(LN)=XB(LN)
      PXC(LN)=PXB(LN)
115   CONTINUE
             
58    BETA=0.0
      DO 90 LJ=1,NM
      XND=SB(LJ)*CFI
      YND=SB(LJ)*SFI
      ZND=CB(LJ)

      XND1=REA(1,1)*XND+REA(1,2)*YND+REA(1,3)*ZND
      YND1=REA(2,1)*XND+REA(2,2)*YND+REA(2,3)*ZND
      ZND1=REA(3,1)*XND+REA(3,2)*YND+REA(3,3)*ZND

      DO 116 MK=1,3
      DL(MK)=(0.0,0.0)
      DR(MK)=(0.0,0.0)
116   CONTINUE

        DO 117 ML=1,ITOT
        DEX=-YIK*(XND1*OX(ML)+YND1*OY(ML)+ZND1*OZ(ML))
        DEX=CEXP(DEX)
        MV=3*(ML-1)+1
        MW=3*(ML-1)+2
        MX=3*(ML-1)+3
      AL=OAL4(ML,1,1)*XC(MV)+OAL4(ML,1,2)*XC(MW)+OAL4(ML,1,3)*XC(MX)
      BL=OAL4(ML,2,1)*XC(MV)+OAL4(ML,2,2)*XC(MW)+OAL4(ML,2,3)*XC(MX)
      CL=OAL4(ML,3,1)*XC(MV)+OAL4(ML,3,2)*XC(MW)+OAL4(ML,3,3)*XC(MX)
      AR=OAL4(ML,1,1)*PXC(MV)+OAL4(ML,1,2)*PXC(MW)+OAL4(ML,1,3)*PXC(MX)
      BR=OAL4(ML,2,1)*PXC(MV)+OAL4(ML,2,2)*PXC(MW)+OAL4(ML,2,3)*PXC(MX)
      CR=OAL4(ML,3,1)*PXC(MV)+OAL4(ML,3,2)*PXC(MW)+OAL4(ML,3,3)*PXC(MX)
        DL(1)=DL(1)+AL*DEX
        DL(2)=DL(2)+BL*DEX
        DL(3)=DL(3)+CL*DEX
        DR(1)=DR(1)+AR*DEX
        DR(2)=DR(2)+BR*DEX
        DR(3)=DR(3)+CR*DEX
117     CONTINUE

      PRL1=RAT(1,1)*DL(1)+RAT(1,2)*DL(2)+RAT(1,3)*DL(3)
      PRL2=RAT(2,1)*DL(1)+RAT(2,2)*DL(2)+RAT(2,3)*DL(3)
      PRL3=RAT(3,1)*DL(1)+RAT(3,2)*DL(2)+RAT(3,3)*DL(3)
      PRR1=RAT(1,1)*DR(1)+RAT(1,2)*DR(2)+RAT(1,3)*DR(3)
      PRR2=RAT(2,1)*DR(1)+RAT(2,2)*DR(2)+RAT(2,3)*DR(3)
      PRR3=RAT(3,1)*DR(1)+RAT(3,2)*DR(2)+RAT(3,3)*DR(3)
 
      ELS=PRL1*CB(LJ)*CFI+PRL2*CB(LJ)*SFI-PRL3*SB(LJ)
      PELS=PRR1*CB(LJ)*CFI+PRR2*CB(LJ)*SFI-PRR3*SB(LJ)
      EDS=PRL1*SFI-PRL2*CFI
      PEDS=PRR1*SFI-PRR2*CFI

C     Calculate amplitude scattering matrix elements.
 
      S1=EDS-PEDS
      S2=-CI*(PELS+ELS)
      S3=ELS-PELS
      S4=-CI*(PEDS+EDS)

      if (lj .gt. 1) go to 378
      cexts=0.5*xkf2*(real(s1)+real(s2))*areau+cexts
c      write(6,*) cexts

C     Call to 'MUELL' to calculate scattering matrix elements.

378      CALL MUELL(S1,S2,S3,S4,S)
          S11(LJ)=S11(LJ)+S(1,1)*AREAU
C          S12(LJ)=S12(LJ)+S(1,2)*AREAU
C          S13(LJ)=S13(LJ)+S(1,3)*AREAU
C          S14(LJ)=S14(LJ)+S(1,4)*AREAU
C          S22(LJ)=S22(LJ)+S(2,2)*AREAU
C          S23(LJ)=S23(LJ)+S(2,3)*AREAU
C          S24(LJ)=S24(LJ)+S(2,4)*AREAU
C          S33(LJ)=S33(LJ)+S(3,3)*AREAU
C          S34(LJ)=S34(LJ)+S(3,4)*AREAU
C          S44(LJ)=S44(LJ)+S(4,4)*AREAU
 
      BETA=BETA+DBETA
90    CONTINUE
 
      AFI=AFI+DEFI
      SAREA=SAREA+AREAU
19    CONTINUE
      ATHETA=ATHETA+DTHETA

18    CONTINUE
      ASI=ASI+DSI
      write(6,*) na
17    CONTINUE

      cexts=cexts/sarea
      write(26,*) cexts

C     Calculate angular dependence of the scattering matrix elements.
 
      BETA=0.0
      WRITE(6,*) SAREA
      DO 93 LG=1,NM
      DEGB=BETA*180.0/PI
      S11(LG)=XKF*S11(LG)/SAREA
C      S12(LG)=XKF*S12(LG)
C      S13(LG)=XKF*S13(LG)
C      S14(LG)=XKF*S14(LG)
C      S22(LG)=XKF*S22(LG)
C      S23(LG)=XKF*S23(LG)
C      S24(LG)=XKF*S24(LG)
C      S33(LG)=XKF*S33(LG)
C      S34(LG)=XKF*S34(LG)
C      S44(LG)=XKF*S44(LG)
c      WRITE(24,*) DEGB,S11(LG),S12(LG),S22(LG),S33(LG),S44(LG)
c      WRITE(25,*) DEGB,S13(LG),S14(LG),S23(LG),S24(LG),S34(LG)
      WRITE(26,*) DEGB,S11(LG)
      BETA=BETA+DBETA
93    CONTINUE

      XLF=XLF+XLFINC
379   CONTINUE

      STOP
      END
C
C
C
C
C
      SUBROUTINE EULERT(CF,SF,CS,SS,CT,ST,AT,EA)
      DIMENSION AT(3,3),EA(3,3)
      AT(1,1)=CS*CF-CT*SF*SS
      AT(1,2)=-SS*CF-CT*SF*CS
      AT(1,3)=ST*SF
      AT(2,1)=CS*SF+CT*CF*SS
      AT(2,2)=-SS*SF+CT*CF*CS
      AT(2,3)=-ST*CF
      AT(3,1)=SS*ST
      AT(3,2)=ST*CS
      AT(3,3)=CT
      EA(1,1)=AT(1,1)
      EA(1,2)=AT(2,1)
      EA(1,3)=AT(3,1)
      EA(2,1)=AT(1,2)
      EA(2,2)=AT(2,2)
      EA(2,3)=AT(3,2)
      EA(3,1)=AT(1,3)
      EA(3,2)=AT(2,3)
      EA(3,3)=AT(3,3)
      RETURN
      END
C
C
C
C
C
      SUBROUTINE TENROT(TA,TAT,TEA,TB)
      DIMENSION TA(3,3),TAT(3,3),TEA(3,3)
      DIMENSION TB(3,3),TC(3,3)
      DO 10 I=1,3
      DO 20 J=1,3
      TC(I,J)=0.0
      DO 30 K=1,3
      PRD1=TA(I,K)*TEA(K,J)
      TC(I,J)=TC(I,J)+PRD1
30    CONTINUE
20    CONTINUE
10    CONTINUE
      DO 40 L=1,3
      DO 50 M=1,3
      TB(L,M)=0.0
      DO 60 N=1,3
      PRD2=TAT(L,N)*TC(N,M)
      TB(L,M)=TB(L,M)+PRD2
60    CONTINUE
50    CONTINUE
40    CONTINUE
      RETURN
      END
C
C
C
C
C
      SUBROUTINE TANG(YM,YM2,CA,SA,CKA,SKA,DT)
      DIMENSION DT(3)
      COMMON /DATA/R,P,IH1,RSM,PSM,IH2
      PI=3.14159265376
      TURNS=YM2/PSM
      B1=-CKA*SA-TURNS*CA*SKA
      B2=SKA*CA+TURNS*SA*CKA
      B3=CKA*CA-TURNS*SA*SKA
      B4=-SKA*SA+TURNS*CA*CKA
      PPM=P/(2.0*PI*YM)
      E1=RSM*B1-IH1*IH2*RSM*B2*PPM-R*SA
      E2=RSM*IH1*B3+IH2*RSM*B4*PPM+IH1*R*CA
      E3=-IH2*IH1*RSM*TURNS*(R/YM)*CKA+P/(2.0*PI)
      A=SQRT(E1**2+E2**2+E3**2)
      DT(1)=E1/A
      DT(2)=E2/A
      DT(3)=E3/A
      RETURN
      END
C
C
C
C
C
      SUBROUTINE MUELL(S1,S2,S3,S4,S)
      COMPLEX S1,S2,S3,S4
      COMPLEX S1C,S2C,S3C,S4C
      DIMENSION S(4,4)
      S1C=CONJG(S1)
      S2C=CONJG(S2)
      S3C=CONJG(S3)
      S4C=CONJG(S4)
      S1Q=S1*S1C
      S2Q=S2*S2C
      S3Q=S3*S3C
      S4Q=S4*S4C
      S(1,1)=0.5*(S1Q+S2Q+S3Q+S4Q)
      S(1,2)=0.5*(S2Q-S1Q+S4Q-S3Q)
      S(1,3)=REAL(S2*S3C+S1*S4C)
      S(1,4)=AIMAG(S2*S3C-S1*S4C)
      S(2,1)=0.5*(S2Q-S1Q-S4Q+S3Q)
      S(2,2)=0.5*(S2Q+S1Q-S4Q-S3Q)
      S(2,3)=REAL(S2*S3C-S1*S4C)
      S(2,4)=AIMAG(S2*S3C+S1*S4C)
      S(3,1)=REAL(S2*S4C+S1*S3C)
      S(3,2)=REAL(S2*S4C-S1*S3C)
      S(3,3)=REAL(S1*S2C+S3*S4C)
      S(3,4)=AIMAG(S2*S1C+S4*S3C)
      S(4,1)=AIMAG(S2C*S4+S3C*S1)
      S(4,2)=AIMAG(S2C*S4-S3C*S1)
      S(4,3)=AIMAG(S1*S2C-S3*S4C)
      S(4,4)=REAL(S1*S2C-S3*S4C)
      RETURN
      END
